package extract;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;

public class AnimFile {
	ByteBuffer dataBuffer;
	int[] offsets;
	ArrayList[] anims;
	
	public AnimFile(byte[] mb)
	{
		dataBuffer = ByteBuffer.wrap(mb);
		parseFile();
	}
	AnimFile(ByteBuffer mb)
	{
		dataBuffer = mb;
		parseFile();
	}
	public int getNumAnims()
	{
		return offsets.length;
	}
	@SuppressWarnings("unchecked")
	void parseFile()
	{
		dataBuffer.order(ByteOrder.LITTLE_ENDIAN);
		int lastIndex = dataBuffer.getInt();
		
		offsets = new int[lastIndex/4];
		offsets[0] = lastIndex;
		for(int n = 1;n < offsets.length;n++)
		{
			offsets[n] = dataBuffer.getInt();
		}
		anims = new ArrayList[offsets.length-1];
		for(int n = 0;n < offsets.length;n++)
		{
			if(offsets[n] == 0) break;
			dataBuffer.position(offsets[n]);
			System.out.println("Now at position - "+offsets[n]+", "+dataBuffer.position());
			anims[n] = new ArrayList<FramePair>();
			int tell,index,time;
			boolean end = false;
			boolean specialblock = false;
			boolean loopflag = false;
			while(!end)
			{
				if(dataBuffer.position() == offsets[n+1])
					break;
				if(dataBuffer.remaining()<= 0)
					break;
				tell = dataBuffer.getChar();
				switch(tell)
				{
					//case 65329:
					case 1:
						index = dataBuffer.getInt();
						time = dataBuffer.getChar();
						anims[n].add(new FramePair(index,time));
						break;
					case 2: 
						loopflag = true;
					case 3:
						if(loopflag)
							end=true;
						else
							System.err.println("Found loop end without loopstart! "+dataBuffer.position());
						break;
					case 4:
						if(!loopflag)
							end=true;
						else
							System.err.println("Found regular end with loopstart! "+dataBuffer.position());
						break;
					case 8:
						specialblock = true;
						dataBuffer.getShort();
						break;
					case 65324:
						if(specialblock)
							specialblock = false;
						else
							System.err.println("Found a strange block! "+dataBuffer.position());
						break;
					/*case 7:
					case 8:
					case 22:
					case 58:
						dataBuffer.getShort();
						break;
					case 13:
						dataBuffer.getInt();
						break;
					case 65303:
					case 65318:
						dataBuffer.getInt();
						dataBuffer.getShort();
					case 23:
					case 24:
					case 394:
					case 514:
					case 2858:
					case 65304:
					case 65324:
					case 65315:
					case 770:
						break;*/
					default:
						System.out.println(n+"  Found a type "+tell+" at position "+dataBuffer.position());
				}
			}
		}
	}
}

class FramePair
{
	protected int sprite,time;
	FramePair(int n, int t)
	{
		sprite = n;
		time = t;
	}
}