package extract;

import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferByte;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class BKTile {
	final static int tileSize = 16;
	public static int[] getTile(BufferedImage bi, int index)
	{
		int sizeX = bi.getWidth()/tileSize;
		int sizeY = bi.getHeight()/tileSize;
		if(index >= sizeX*sizeY)
			return null;
		int ytile = index%sizeY;
		int xtile = index/sizeY;
		int[] rgbArray = new int[256];
		bi.getRGB(xtile*tileSize, ytile*tileSize, tileSize, tileSize, rgbArray, 0, tileSize);
		return rgbArray;
		//return bi.getData(new Rectangle(xtile*tileSize, ytile*tileSize, tileSize, tileSize));
		//return bi.getSubimage(xtile*tileSize, ytile*tileSize, tileSize, tileSize);
	}
	public static int[] flipTileHorizontal(int[] source)
	{
		int[] dest = new int[source.length];
		for(int n = 0;n < 16;n++)
		{
			for(int m = 0;m < 16;m++)
			{
				dest[15-m+n*16] = source[m+n*16];
			}
		}
		return dest;
	}
	public static int[] flipTileVertical(int[] source)
	{
		int[] dest = new int[source.length];
		for(int n = 0;n < 16;n++)
		{
			for(int m = 0;m < 16;m++)
			{
				dest[m+(15-n)*16] = source[m+n*16];
			}
		}
		return dest;
	}
	public static int[] flipTileBoth(int[] source)
	{
		int[] dest = new int[source.length];
		for(int n = 0;n < 16;n++)
		{
			for(int m = 0;m < 16;m++)
			{
				dest[(15-m)+(15-n)*16] = source[m+n*16];
			}
		}
		return dest;
	}
	public static void setTile(BufferedImage dest, int[] src, int index)
	{
		int sizeX = dest.getWidth()/tileSize;
		int sizeY = dest.getHeight()/tileSize;
		if(index >= sizeX*sizeY)
		{
			System.err.println("Tile index out of range! "+index+", "+sizeX*sizeY);
			return;
		}
		int ytile = index%sizeY;
		int xtile = index/sizeY;
		dest.setRGB(xtile*tileSize, ytile*tileSize, tileSize, tileSize, src, 0, tileSize);
		//g.drawImage(src,xtile*tileSize, ytile*tileSize,null);
	}
	public static BufferedImage makeTileSheet(ByteBuffer bb, IndexColorModel icm,boolean flip)
	{
		bb.order(ByteOrder.LITTLE_ENDIAN);
		int width = bb.getInt();
		int height = bb.getInt();
		int colorDepth = bb.getShort();
		if(colorDepth != 8)
		{
			System.err.println("Color Depth Mismatch!");
		}
		byte[] imgData = new byte[width*height];
		if(flip)
		{
			for(int n = height-1;n >= 0;n--)
			{
				bb.get(imgData, n*width, width);
			}
		} else {
			bb.get(imgData);
		}
		DataBuffer db = new DataBufferByte(imgData, width*height); 
		WritableRaster raster = Raster.createWritableRaster(icm.createCompatibleSampleModel(width,height), db, null);
		return new BufferedImage(icm, raster, false, null);
	}
	static BufferedImage makePaletteImage(IndexColorModel icm)
	{
		byte[] imgData = new byte[256*256];
		for(int n = 0;n < 256;n++)
		{
			for(int x=0;x < 16;x++)
			{
				for(int y=0;y < 16;y++)
				{
					int xcoord = n%16*16;
					int ycoord = (n/16);
					imgData[xcoord+x+(ycoord*16+y)*256] = (byte)n;
				}
			}
		}
		DataBuffer db = new DataBufferByte(imgData, 256*256); 
		WritableRaster raster = Raster.createWritableRaster(icm.createCompatibleSampleModel(256,256), db, null);
		return new BufferedImage(icm, raster, false, null);
	}
}
