package extract;

import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferByte;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;

public class BKTiles {
	HashMap<Integer,BufferedImage> tilemap = new HashMap<Integer,BufferedImage>();;
	static IndexColorModel palette;
	
	void makeFakePalette()
	{
		byte[] paldata = new byte[64];
		for(int n = 0;n < 16;n++)
		{
			paldata[n*4] = (byte) (n*16);
			paldata[n*4+1] = (byte) (n*16);
			paldata[n*4+2] = (byte) (n*16);
			paldata[n*4+3] = (byte) (0xff);
		}
		palette = new IndexColorModel(8,16,paldata,0,true,0);
	}
	public IndexColorModel getPalette()
	{
		return palette;
	}
	public BKTiles(PakExtract tiles)
	{
		makeFakePalette();
		for(Integer n: tiles.getKeys())
		{
			tilemap.put(n, makeSheet(tiles.getFile(n)));
		}
	}
	public BufferedImage getSheet(int index)
	{
		return tilemap.get(new Integer(index));
	}
	BufferedImage makeSheet(byte[] data)
	{
		ByteBuffer bb = ByteBuffer.wrap(data);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		int width = bb.getChar();
		int height = bb.getChar();
		int bitdepth = bb.getInt();
		int dataSize = bb.getInt();
		if((data.length - 16) != dataSize)
		{
			System.err.println("Data size mismatch detected.");
		}
		int unknown = bb.getInt();
		DataBuffer db;
		switch(bitdepth)
		{
			case 4:
				byte[] rasterData = new byte[width*height];
				int position = 0;
				while(bb.remaining() > 0)
				{
					byte value = bb.get();
					rasterData[position++] = (byte) (value&0xf);
					rasterData[position++] = (byte) ((value&0xf0)>>4);
				}
				db = new DataBufferByte(rasterData, width*height);
				break;
			default:
				db = new DataBufferByte(data, width*height,16);
		}
		WritableRaster raster = Raster.createWritableRaster(palette.createCompatibleSampleModel(width,height), db, null);
		return new BufferedImage(palette, raster, false, null);
	}
}
