package extract;

import java.awt.Rectangle;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class BoxFile {
	ByteBuffer dataBuffer;
	int[] offsets;
	
	public BoxFile(byte[] mb)
	{
		dataBuffer = ByteBuffer.wrap(mb);
		parseFile();
	}
	BoxFile(ByteBuffer mb)
	{
		dataBuffer = mb;
		parseFile();
	}
	void parseFile()
	{
		dataBuffer.order(ByteOrder.LITTLE_ENDIAN);
		int lastIndex = dataBuffer.getInt();
		
		offsets = new int[lastIndex/4];
		offsets[0] = lastIndex;
		for(int n = 1;n < offsets.length;n++)
		{
			offsets[n] = dataBuffer.getInt();
		}
	}
	public Rectangle[] getBoxes(int index)
	{
		if(offsets[index] == 0)
		{
			return null;
		}
		dataBuffer.position(offsets[index]);
		int type1 = dataBuffer.get();
		int type2 = dataBuffer.get();
		int type3 = dataBuffer.get();
		int type4 = dataBuffer.get();
		int pos = 0;
		
		Rectangle[] boxes = new Rectangle[type1+type3+type4];
		for(int n = 0;n < type1;n++)
		{
			int x = dataBuffer.getShort();
			int y = dataBuffer.getShort();
			int width = dataBuffer.getShort();
			int height = dataBuffer.getShort();
			boxes[pos++] = new Rectangle(x,y,width,height);
		}
		for(int n = 0;n < type2;n++)
		{
			int x = dataBuffer.getShort();
			int y = dataBuffer.getShort();
			int width = dataBuffer.getShort();
			int height = dataBuffer.getShort();
			//boxes[pos++] = new Rectangle(x,y,width,height);
		}
		for(int n = 0;n < type3;n++)
		{
			int x = dataBuffer.getShort();
			int y = dataBuffer.getShort();
			int width = dataBuffer.getShort();
			int height = dataBuffer.getShort();
			boxes[pos++] = new Rectangle(x,y,width,height);
		}
		for(int n = 0;n < type4;n++)
		{
			int x = dataBuffer.getShort();
			int y = dataBuffer.getShort();
			int width = dataBuffer.getShort();
			int height = dataBuffer.getShort();
			boxes[pos++] = new Rectangle(x,y,width,height);
		}
		return boxes;
	}
	public Rectangle[] getAttackBoxes(int index)
	{
		if(offsets[index] == 0)
		{
			return null;
		}
		dataBuffer.position(offsets[index]);
		int type1 = dataBuffer.get();
		int type2 = dataBuffer.get();
		int type3 = dataBuffer.get();
		int type4 = dataBuffer.get();
		int pos = 0;
		
		Rectangle[] boxes = new Rectangle[type2];
		for(int n = 0;n < type1;n++)
		{
			int x = dataBuffer.getShort();
			int y = dataBuffer.getShort();
			int width = dataBuffer.getShort();
			int height = dataBuffer.getShort();
			//boxes[pos++] = new Rectangle(x,y,width,height);
		}
		for(int n = 0;n < type2;n++)
		{
			int x = dataBuffer.getShort();
			int y = dataBuffer.getShort();
			int width = dataBuffer.getShort();
			int height = dataBuffer.getShort();
			boxes[pos++] = new Rectangle(x,y,width,height);
		}
		return boxes;
	}
}
