package extract;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.FileChannel.MapMode;
import java.util.HashMap;
import java.util.Set;

public class PakExtract {
	HashMap<Integer,SubFile> filemap;
	final static String magic = new String(new byte[]{0x44,0x41,0x54,0x41});
	final static String magic2 = new String(new byte[]{0x43,0x48,0x52,0x44});
	ByteBuffer dataBuffer;
	
	public void setBuffer(ByteBuffer b)
	{
		dataBuffer = b;
		//files = new ArrayList<SubFile>();
		filemap = new HashMap<Integer,SubFile>();
		extract(dataBuffer);
	}
	Set<Integer> getKeys()
	{
		return filemap.keySet();
	}
	public byte[] getFile(Integer n)
	{
		if(dataBuffer != null)
			return filemap.get(n).get(dataBuffer);
		return null;
	}
	protected void extract(ByteBuffer b)
	{
		b.order(ByteOrder.LITTLE_ENDIAN);
		byte[] check = new byte[4];
		b.get(check);
		if(!magic.equals(new String(check)) && !magic2.equals(new String(check)))
		{
			System.err.println("Invalid File.");
		}
		b.getInt(); // Character index?
		b.getInt();
		int numdats = b.getInt();
		int start = -1;
		int end = -1;
		int index = 0;
		for(int n = 0;n <= numdats;n++)
		{
			int value = b.getInt();
			if(value != 0)
			{
				start = end;
				end = value;
				
				if(start > 0)
				{
					if(end > b.limit())
						end = b.limit();
					filemap.put(new Integer(index), new SubFile(index,start,end-start));
				}
				index = n;
			}
		}
	}
	static void saveFile(byte[] data, File f)
	{
		try {
			f.getParentFile().mkdirs();
			FileOutputStream fos = new FileOutputStream(f);
			fos.write(data);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public static void main(String args[])throws IOException
	{
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		System.out.println("File path?");
		String path = br.readLine();
		File f = new File(path);
		if(!f.exists()){System.exit(0);}
		PakExtract p = new PakExtract();
		FileChannel fc = new FileInputStream(f).getChannel();
		MappedByteBuffer mb = fc.map(MapMode.READ_ONLY, 0, fc.size());
		p.setBuffer(mb);
		String filename = f.getName();
		filename = filename.substring(0,filename.indexOf('.'));
		for(Integer n: p.getKeys())
		{
			File save = new File(String.format("out/%s%03d.bin", filename,n));
			saveFile(p.getFile(n),save);
		}
	}
}

class SubFile
{
	int index, offset, length;
	SubFile(int n, int o, int l)
	{
		index = n;
		offset = o;
		length = l;
	}
	byte[] get(ByteBuffer b)
	{
		byte[] returnval = new byte[length];
		b.position(offset);
		b.get(returnval);
		return returnval;
	}
}
