package extract;

import java.awt.image.BufferedImage;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class TileFile {
	int[] offsets;
	ByteBuffer dataBuffer;
	final static int tileSize = 16;
	
	public TileFile(byte[] mb)
	{
		dataBuffer = ByteBuffer.wrap(mb);
		parseFile();
	}
	TileFile(ByteBuffer mb)
	{
		dataBuffer = mb;
		parseFile();
	}
	void parseFile()
	{
		dataBuffer.order(ByteOrder.LITTLE_ENDIAN);
		int lastIndex = dataBuffer.getInt();
		
		offsets = new int[lastIndex/4];
		offsets[0] = lastIndex;
		for(int n = 1;n < offsets.length;n++)
		{
			offsets[n] = dataBuffer.getInt();
		}
	}
	public int getNumSprites()
	{
		return offsets.length;
	}
	public boolean spriteExists(int index)
	{
		if(offsets[index] == 0)
			return false;
		dataBuffer.position(offsets[index]);
		
		return (dataBuffer.getShort()!=0);
	}
	public int getXsize(int index)
	{
		dataBuffer.position(offsets[index]+4);
		return dataBuffer.getChar()*16;
	}
	public int getYsize(int index)
	{
		dataBuffer.position(offsets[index]+6);
		return dataBuffer.getChar()*16;
	}
	public int getYaxis(int index)
	{
		dataBuffer.position(offsets[index]+8);
		return dataBuffer.getShort();
	}
	public int getXaxis(int index)
	{
		dataBuffer.position(offsets[index]+10);
		return dataBuffer.getShort();
	}
	public BKSprite getMeta(int index)
	{
		BKSprite returnval = new BKSprite();
		dataBuffer.position(offsets[index]);
		
		dataBuffer.getShort();
		while(true)
		{
			if(dataBuffer.remaining() < 12)
				return returnval;
			else if(dataBuffer.position()>= offsets[index+1])
			{
				return returnval;
			}
		dataBuffer.getShort(); //Unknown Value
		int tilesX = dataBuffer.getChar();
		int tilesY = dataBuffer.getChar();
		int offY = dataBuffer.getShort();
		int offX = dataBuffer.getShort();
		byte[] empty = new byte[tilesX*tilesY*4];
		dataBuffer.get(empty);
		BufferedImage result = new BufferedImage(tilesX*tileSize, tilesY*tileSize, BufferedImage.TYPE_INT_ARGB);
		returnval.addImage(result, offX, offY);
		}
	}
	public BKSprite buildImage(BKTiles tiles,int index)
	{
		BKSprite returnval = new BKSprite();
		dataBuffer.position(offsets[index]);
		
		dataBuffer.getShort();
		while(true)
		{
			if(dataBuffer.remaining() < 12)
				return returnval;
			else if(dataBuffer.position()>= offsets[index+1])
			{
				return returnval;
			}
		dataBuffer.getShort(); //Unknown Value
		int tilesX = dataBuffer.getChar();
		int tilesY = dataBuffer.getChar();
		int offY = dataBuffer.getShort();
		int offX = dataBuffer.getShort();
		BufferedImage result = new BufferedImage(tilesX*tileSize, tilesY*tileSize, BufferedImage.TYPE_BYTE_INDEXED, tiles.getPalette());

		int currentSheetIndex = -1;
		BufferedImage currentSheet = null;
		
		int pos = 0;
		for(int x = 0; x < tilesX; x++)
		{
			for(int y = 0; y < tilesY; y++)
			{
				int tileIndex = dataBuffer.get()&0xff;
				int sheetIndex = dataBuffer.get()&0xff;
				byte flip = dataBuffer.get();
				sheetIndex |= ((flip&0xf0)<<4);
				flip = (byte) (flip&0x0f);
				byte unknown = dataBuffer.get();
				if(tileIndex == 0 && sheetIndex == 1)
				{
					pos++;
					continue;
				}
				if(sheetIndex != currentSheetIndex)
				{
					currentSheetIndex = sheetIndex;
					currentSheet = tiles.getSheet(sheetIndex);
				}
				int[] sourceTile = BKTile.getTile(currentSheet, tileIndex);
				switch(flip)
				{
					case 3: sourceTile = BKTile.flipTileBoth(sourceTile);break;
					case 1: sourceTile = BKTile.flipTileHorizontal(sourceTile);break;
					case 2: sourceTile = BKTile.flipTileVertical(sourceTile);break;
					default:
				}
				BKTile.setTile(result, sourceTile, pos++);
			}
		}
		returnval.addImage(result, offX, offY);
		}
		//return returnval;
	}
	public int[] getIdealSize()
	{
		int lowx=0,lowy=0,highx=0,highy=0;
		for(int n = 0;n < offsets.length;n++)
		{
			if(!spriteExists(n))
				continue;
			BKSprite s = getMeta(n);
			for(int m = 0;m < s.getNumImages();m++)
			{
				int ax = s.getOffX(m);
				int ay = s.getOffY(m);
				int w = s.getImage(m).getWidth();
				int h = s.getImage(m).getHeight();
				if(ax < lowx)
					lowx = ax;
				if(ay < lowy)
					lowy = ay;
				if(ax+w > highx)
					highx = ax+w;
				if(ay+h > highy)
					highy = ay+h;
			}
		}
		int totalWidth = highx-lowx;
		int totalHeight = highy-lowy;
		int axisx = -1*lowx;
		int axisy = -1*lowy;
		int[] dimensions = {totalWidth,totalHeight,axisx,axisy};
		return dimensions;
	}
}